/*
 * GameBoardLocation.java
 *
 * Created on August 31, 2002, 12:08 AM
 */

/**
 *
 * @author  scerruti
 */
public class GameBoardLocation {
    
    private int x;
    
    private int y;
    
    private GameMap map;
    
    /** Creates a new instance of GameBoardLocation */
    public GameBoardLocation(GameMap map, int x, int y) {
        super();
        this.map = map;
        this.x = x;
        this.y = y;
    }

    public GameBoardLocation(GameMap map) {
        this(map, 0, 0);
    }

    void setLocation(int x, int y) throws BadLocationException {
        if (x < 1 || y < 1 || x > map.getWidth() || y > map.getHeight()) {
            throw new BadLocationException();
        }
        this.x = x;
        this.y = y;
    }
    
    int getX() {
        return (this.x);
    }
    
    int getY() {
        return (this.y);
    }
    
    GameMap getMap() {
        return (this.map);
    }
    
    int distanceTo(GameBoardLocation otherLocation) {
        return (Math.abs(this.x - otherLocation.getX()) +
                Math.abs(this.y - otherLocation.getY()));
    }
    
    void moveEast() throws BadLocationException {
        if (this.x >= map.getWidth()) throw new BadLocationException();
        this.x++;
    }
    
    void moveWest() throws BadLocationException {
        if (this.x <= 1) throw new BadLocationException();
        this.x--;
    }
    
    void moveNorth() throws BadLocationException {
        if (this.y >= map.getHeight()) throw new BadLocationException();
        this.y++;
    }
    
    void moveSouth() throws BadLocationException {
        if (this.y <= 1) throw new BadLocationException();
        this.y--;
    }
    
    boolean equals(GameBoardLocation loc) {
        return (loc.getX() == this.x && loc.getY() == this.y);
    }
    
    void dump() {
        System.out.println("X: "+x+" Y: "+y);
    }
}

