/*
 * Main.java
 *
 * Created on August 30, 2002, 9:54 PM
 */

import java.io.*;
import java.util.*;

/**
 *
 * @author  scerruti
 */
public class JPackageBot {
    
    RobotServer robotServer;
    GameMap gameMap;
    MyPkgRobot myRobot;
    HashSet allRobots;
    
    /** Creates a new instance of Main */
    public JPackageBot(String server, int port) {
        //System.out.println("server: "+server+" port: "+port);
        
        // Open Connection To Server
        try {
            robotServer = new RobotServer(server,port);
        } catch (IOException e) {
            System.err.println ("JPackageBot: can't connect to server "+
                e.getMessage());
            System.exit(1);
        }
        
        // Initialize Session
        robotServer.startGame();
        
        // Get Game Map
        try {
            gameMap = robotServer.getMap();
        } catch (IOException e) {
            System.out.println ("JPackageBot: can't get map "+
                e.getMessage());
            System.exit(1);
        } catch (BadTileTypeException e) {
            System.out.println ("JPackageBot: bad map tile "+
                e.getMessage());
            System.exit(1);
        } catch (BadLocationException e) {
            System.out.println ("JPackageBot: bad map location "+
                e.getMessage());
            System.exit(1);
        }
        //gameMap.dump();
         
        // Get My Robot Configuration
        try {
            myRobot = robotServer.getMyRobot();
        } catch (IOException e) {
            System.out.println ("JPackageBot: can't get my robot "+
                e.getMessage());
            System.exit(1);
        }       
        
        // Get Other Robot Locations
        try {
            allRobots = robotServer.getRobotLocations();
        } catch (IOException e) {
            System.out.println ("JPackageBot: can't get robot locations "+
                e.getMessage());
            System.exit(1);
        } catch (BadLocationException e) {
            System.out.println ("JPackageBot: bad robot location "+
                e.getMessage());
            System.exit(1);
        }       
        
        if (allRobots.isEmpty()) {
            System.out.println ("JPackageBot: no robots found");
            System.exit(1);
        }
        
        Iterator robot_iterator =  allRobots.iterator();
        while (robot_iterator.hasNext()) {
            PkgRobot robot = (PkgRobot) robot_iterator.next();
            if (robot.getId() == myRobot.getId()) {
                myRobot.setLocation(robot.getLocation());
                robot_iterator.remove();
            } else {
                gameMap.addRobot(robot);
                robot_iterator.remove();
            }
        }
        
        // Choose 
        
        
        while (robotServer.connectionOpen()) {
            try {
                TreeSet pkgs = robotServer.getPkgs();
                if (robotServer.connectionOpen() == false) break;
                
                robotServer.issueCommand(myRobot.takeTurn(pkgs));
                pkgs=null;
                ArrayList srvResp = robotServer.getResponse();
                
                Iterator i = srvResp.iterator();
                while (i.hasNext()) {
                    RobotServerCommand rsc = (RobotServerCommand) i.next();
                    
                    if (rsc.id == myRobot.getId()) {
                        myRobot.doCommand(rsc);
                    } else {
                        gameMap.doRobotCommand(rsc);
                    }
                }
                gameMap.cleanRobots();
                myRobot.resetKill();
                //gameMap.dump();
                //myRobot.dump();
            } catch (Exception e) {
                e.printStackTrace(System.err);
               
                //robotServer.disconnect();
                //myRobot.shutdown();
            }
        }
        
        try {
            //myRobot.dump();
        } catch (Exception e) {
            System.out.println(e.getMessage());
            System.exit(1);
        }
        
        // Exit Normally
        System.exit(0);
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        int port=-1;
        
        if (args.length != 2) {
            System.out.println("usage: JPackageBot <server> <port>");
            System.exit(1);
        }
        
        try {
            port = Integer.parseInt(args[1]);
        } catch (NumberFormatException e) {     
            System.out.println("JPackageBot: invalid server port "+e.getMessage());
            System.exit(1);
        }
        
        new JPackageBot(args[0], port);
    }
    
}
