/*
 * MapSolver.java
 *
 * Created on August 31, 2002, 12:13 PM
 */

import java.lang.*;
import java.util.*;

/**
 *
 * @author  scerruti
 */
public class MapSolver {
    
    private GameMap map;
    private MyPkgRobot me;
    private char direction;
    private int pathLen;
    
    public static final char SOUTH = 'S';
    public static final char NORTH = 'N';
    public static final char EAST  = 'E';
    public static final char WEST  = 'W';
    
    /** Creates a new instance of MapSolver */
    public MapSolver(MyPkgRobot me) {
        super();
        this.map = me.getLocation().getMap();
        this.me  = me;
        this.run((int) (Math.max(map.getWidth(),map.getHeight())/10));
    }
    
    public void run(int buffer) {
                
        // Get Starting Location
        GameBoardLocation start_loc = me.getLocation();
        this.pathLen = Integer.MAX_VALUE;
            
        // Determine Possible Destinations
        TreeSet topDestinations = getPriorityDestinations();
        //System.out.println(topDestinations.size());
        // Determine route length to each destination
        Iterator i = topDestinations.iterator();
        while (i.hasNext()) {
            GameBoardLocation loc = (GameBoardLocation) i.next();
             
            int gridMinX = Math.max(1,
                Math.min(loc.getX()-buffer,start_loc.getX()-buffer));
            int gridMaxX = Math.min(map.getWidth(), 
                Math.max(loc.getX()+buffer,start_loc.getX()+buffer));
            int gridMinY = Math.max(1,
                Math.min(loc.getY()-buffer,start_loc.getY()-buffer));
            int gridMaxY = Math.min(map.getHeight(),
                Math.max(loc.getY()+buffer,start_loc.getY()+buffer));
             
            Flood myFlood = new Flood(map, start_loc, loc,
                gridMinX, gridMinY, gridMaxX, gridMaxY);
            if (this.pathLen > myFlood.floodLen &&
                me.backward() != myFlood.floodMove) {
                this.direction = myFlood.floodMove;
                this.pathLen   = myFlood.floodLen;
            }
        }
        if (this.pathLen == Integer.MAX_VALUE) {
            if (buffer >= Math.max(map.getWidth(),map.getHeight())) {
                    this.direction = NORTH; // Probably into oblivion
                    return;
            } else {
                this.run(buffer*2);
            }
        }
    }
    
    public char getDirection() {
        return (direction);
    }
        
    private TreeSet getPriorityDestinations() {
        TreeSet destinations = new TreeSet(new SortedDest(me.getLocation()));
        
        // Not the best algorithm
        // If we have packages, go to the nearest destination
        // otherwise head for the nearest non-empty base
        if (me.myPkgs != null && me.myPkgs.size() > 0) {
            Iterator pkgs = me.myPkgs.iterator();
            while (pkgs.hasNext()) {
                GamePkg pkg = (GamePkg) pkgs.next();
                destinations.add(pkg.getLocation());
                //System.out.print("Package: ");
                //pkg.getLocation().dump();
            }
            return (destinations);
        }
        
        Iterator bases = map.getBases().iterator();
        //System.out.println("# bases: "+map.getBases().size());
        if (map.getBases().size() > 0) {
            while (bases.hasNext()) {
                GameBoardLocation base = (GameBoardLocation) bases.next();
                destinations.add(base);
                //System.out.print("Base: ");
                //base.dump();
            }
        } else {
            // No destination? Attack the closest robot!
            Iterator i = map.getRobots().entrySet().iterator();
            while (i.hasNext()) {
                Map.Entry entry = (Map.Entry) i.next();
                PkgRobot bot = (PkgRobot) entry.getValue();
                destinations.add(bot.getLocation());
                //System.out.print("Bot: ");
                //try {bot.dump();} catch (BadLocationException e) {};
            }
        }

        return (destinations);
    }
    
    private class SortedDest implements Comparator {
        private GameBoardLocation start;
        
        public SortedDest (GameBoardLocation start) {
            super();
            this.start = start;
        }
        
        public int compare (Object one, Object two) {
            int oneDist = ((GameBoardLocation) one).distanceTo(start);
            int twoDist = ((GameBoardLocation) two).distanceTo(start);
            return (twoDist - oneDist);
        }
        /**
        public boolean equals (Object o) {
            return false;
        }*/
    }

    public class Flood {
        int[][] floodMap;
        int xSize;
        int ySize;
        int floodLen;
        char floodMove;

        public  Flood (GameMap map, GameBoardLocation start, 
            GameBoardLocation end,
            int minX, int minY, int maxX, int maxY) {
                    
            xSize = maxX - minX + 1;
            ySize = maxY - minY + 1;
             //System.out.println("Min X:"+minX+" Min Y:"+minY+" Max X:"+maxX+" Max Y:"+maxY);
            int xSize = maxX - minX + 1;
            int ySize = maxY - minY + 1;
            //System.out.println("X Size:"+xSize+" Y Size:"+ySize);
       
            int xLocalStart = start.getX() - minX;
            int yLocalStart = start.getY() - minY;
            int xLocalEnd   = end.getX()   - minX;
            int yLocalEnd   = end.getY()   - minY;
        
            HashSet frontier =new HashSet();        
        
            floodMap = new int[xSize][ySize];
            for (int i = 0; i < xSize; i++) {
                for (int j = 0; j < ySize; j++) {
                    floodMap[i][j] = Integer.MAX_VALUE;
                }
            }
        
            floodMap[xLocalEnd][yLocalEnd] = 0;
        
            // Initialize Frontier
            frontier.add(new Pair(xLocalEnd, yLocalEnd));

            while (!frontier.isEmpty()) {
                HashSet nextFrontier = new HashSet();
                Iterator i = frontier.iterator();
                while (i.hasNext()) {
                    Pair p = (Pair) i.next();
                    int basevalue = floodMap[p.x][p.y] + 1;

                    int mapX = p.x + minX;
                    int mapY = p.y + minY;
                
                    // East
                    if ((p.x > 0) && (floodMap[p.x-1][p.y] > basevalue)) {
                        char type = map.tileTypeAt(mapX-1,mapY);
                        if (type == GameMap.NORMAL || type == GameMap.BASE) {
                            int danger = computeDanger(mapX-1, mapY);
                        
                            if (floodMap[p.x-1][p.y] > (basevalue+danger)) {
                                floodMap[p.x-1][p.y] = basevalue+danger;
                                if (basevalue+danger < floodMap[xLocalStart][yLocalStart]) {
                                    nextFrontier.add(new Pair(p.x-1,p.y));
                                }
                            }
                        }
                    }

                    // West
                    if ((p.x < xSize-1) && (floodMap[p.x+1][p.y] > basevalue)) {
                        char type = map.tileTypeAt(mapX+1,mapY);
                        if (type == GameMap.NORMAL || type == GameMap.BASE) {
                            int danger = computeDanger(mapX+1, mapY);
                        
                            if (floodMap[p.x+1][p.y] > (basevalue+danger)) {
                                floodMap[p.x+1][p.y] = basevalue+danger;
                                if (basevalue+danger < floodMap[xLocalStart][yLocalStart]) {
                                    nextFrontier.add(new Pair(p.x+1,p.y));
                                }
                            }
                        }
                    }

                    // South
                    if ((p.y > 0) && (floodMap[p.x][p.y-1] > basevalue)) {
                        char type = map.tileTypeAt(mapX,mapY-1);
                        if (type == GameMap.NORMAL || type == GameMap.BASE) {
                            int danger = computeDanger(mapX-1, mapY);
                        
                            if (floodMap[p.x][p.y-1] > (basevalue+danger)) {
                                floodMap[p.x][p.y-1] = basevalue+danger;
                                if (basevalue+danger < floodMap[xLocalStart][yLocalStart]) {
                                    nextFrontier.add(new Pair(p.x,p.y-1));
                                }
                            }
                        }
                    }

                    // North
                    if ((p.y < ySize-1) && (floodMap[p.x][p.y+1] > basevalue)) {
                        char type = map.tileTypeAt(mapX,mapY+1);
                        if (type == GameMap.NORMAL || type == GameMap.BASE) {
                            int danger = computeDanger(mapX, mapY+1);
                        
                            if (floodMap[p.x][p.y+1] > (basevalue+danger)) {
                                floodMap[p.x][p.y+1] = basevalue+danger;
                                if (basevalue+danger < floodMap[xLocalStart][yLocalStart]) {
                                    nextFrontier.add(new Pair(p.x,p.y+1));
                                }
                            }
                        }
                    }
                }
                
                frontier = nextFrontier;
            }
            
            floodLen  = floodMap[xLocalStart][yLocalStart];
            if (floodLen == Integer.MAX_VALUE) {
                //System.out.println("No solution at this resolution");
                //dump(xSize,ySize,xLocalEnd,yLocalEnd,xLocalStart,yLocalStart);
                return;
            }
            
            int north = Integer.MAX_VALUE;
            int south = Integer.MAX_VALUE;
            int east  = Integer.MAX_VALUE;
            int west  = Integer.MAX_VALUE;
                    
            if (xLocalStart > 0)
                west = floodMap[xLocalStart-1][yLocalStart];
            if(xLocalStart < xSize - 1)
                east = floodMap[xLocalStart+1][yLocalStart];
            if (yLocalStart > 0)
                south = floodMap[xLocalStart][yLocalStart-1];
            if (yLocalStart < ySize - 1) 
                north  = floodMap[xLocalStart][yLocalStart+1];

            int y = Math.min(north,south);
            int x = Math.min(east,west);
            if (x < y) {
                if (x == east) {
                    floodMove=EAST;
                } else {
                    floodMove=WEST;
                }
            } else {
                if (y == north) {
                    floodMove=NORTH;
                } else {
                    floodMove=SOUTH;
                }
            }

            //System.out.println("-- "+floodMove+" "+floodLen+" --");
            //dump(xSize,ySize,xLocalEnd,yLocalEnd,xLocalStart,yLocalStart);
        }
            
        private int computeDanger(int x, int y) {
            int danger = 0;
            danger = danger + map.dangerAt(x-1, y);
            danger = danger + map.dangerAt(x+1, y);
            danger = danger + map.dangerAt(x, y-1);
            danger = danger + map.dangerAt(x, y+1);
            return (danger);
        }
        
        private void dump(int xSize, int ySize, int xLocalEnd, int yLocalEnd, int xLocalStart, int yLocalStart) {
            for (int h = 0; h < ySize; h++) {
                for (int w = 0; w < xSize; w++) {
                    int a=floodMap[w][h];
                    if (w == xLocalEnd && h ==  yLocalEnd) {
                        System.out.print("@");
                    } else if (w == xLocalStart && h == yLocalStart) {
                        System.out.print("^");
                    } else if (a==Integer.MAX_VALUE) {
                        System.out.print(".");
                    } else if (a < 10) {
                        System.out.print(a);
                    } else {
                        System.out.print((char) ('A'+(a-10)));
                    }
                }
                System.out.println();
            }
        }
                
    }
    
    private class Pair {
        int x,y;
        public Pair (int x,int y) {
            this.x=x;
            this.y=y;
        }
    }

}