/*
 * PkgRobot.java
 *
 * Created on August 31, 2002, 12:06 AM
 */

import java.util.*;

/**
 *
 * @author  scerruti
 */
public class MyPkgRobot extends PkgRobot {
    
    private int capacity;
    
    private int budget;
    
    HashSet myPkgs = null;
    
    TreeSet localPkgs;
    
    char lastDirection = 'X';
    
    private static final int RESET  = 0;
    private static final int PRIMED = 1;
    private static final int ACTIVE = 2;
    int killMode = RESET;
    GameBoardLocation killLocation = null;
    
    /** Creates a new instance of PkgRobot */
    public MyPkgRobot(int id, int capacity, int budget) {
        super(id);
        this.capacity = capacity;
        this.budget = budget;
        this.myPkgs = new HashSet();
    }
       
    public String takeTurn(TreeSet pkgList) {
        // This routine is going to have to decide whether to pick, drop or move
        // Additionally it will have to bid
        int bid = 1; // Default bid
        int command = RobotServerCommand.NO_COMMAND;
        HashSet drop=null;
        HashSet pick=null;
        localPkgs = pkgList;
        lastDirection = 'X';
        
        // Special Kill Mode
        //get target
        //herd target towards lethal ground
        
        // Highest priority - drop packages for this destination
        if (this.myPkgs.size() > 0) {
            Iterator i = this.myPkgs.iterator();
            while (i.hasNext()) {
                GamePkg gp = (GamePkg) i.next();
                if (gp.getLocation().equals(super.getLocation())) {
                    if (drop==null) {
                        drop = new HashSet();
                    }
                    drop.add(gp);
                }
            }
        }
        
        if (drop != null) {
            // Base bid on nearest robot neighbors
            String cmdString = bid + " Drop";
            Iterator i = drop.iterator();
            while (i.hasNext()) {
               cmdString = cmdString + " " + ((GamePkg) i.next()).getId();
            }
            return (cmdString);
        }
        
        // Next highest priority is to pick up a package
        int tmp_capacity = capacity;
        if (pkgList != null && capacity > 0) {
            Iterator i = pkgList.iterator();
            while (i.hasNext()) {
                GamePkg gp = (GamePkg) i.next();
                if (tmp_capacity - gp.getWeight() >= 0) {
                    if (pick == null) {
                        pick = new HashSet();
                    }
                    pick.add(gp);
                    tmp_capacity = tmp_capacity - gp.getWeight();
                }
            }
        }
                    
        if (pick != null) {
            // bid is unimportant for pick
            String cmdString = bid + " Pick";
            Iterator i = pick.iterator();
            while (i.hasNext()) {
               cmdString = cmdString + " " + ((GamePkg) i.next()).getId();
            }
            return (cmdString);
        }
        
        // No drop or pick? Move to closest important destination
        // Bid is based on neighbors
        MapSolver mapSol = new MapSolver(this);
        String cmdString = bid + " Move "+ mapSol.getDirection();
        lastDirection = mapSol.getDirection();
        return (cmdString);
    }
        
    public void doCommand(RobotServerCommand rsc) throws BadLocationException {
        switch (rsc.command) {
            case (RobotServerCommand.MOVE_EAST):
                super.getLocation().moveEast();
                break;
            case (RobotServerCommand.MOVE_WEST):
                super.getLocation().moveWest();
                break;
            case (RobotServerCommand.MOVE_NORTH):
                super.getLocation().moveNorth();
                break;
            case (RobotServerCommand.MOVE_SOUTH):
                super.getLocation().moveSouth();
                break;
            case (RobotServerCommand.PICK_PACKAGE):
                pickPkg(rsc.pkg);
                killMode = PRIMED;
                killLocation = this.getLocation();
                break;
            case (RobotServerCommand.DROP_PACKAGE):
                dropPkg(rsc.pkg);
                if (killMode == PRIMED) killMode = ACTIVE;
                break;
            case (RobotServerCommand.SET_LOCATION):
                super.getLocation().setLocation(rsc.x,rsc.y);
                break;
            case (RobotServerCommand.NO_COMMAND):
                break;
        }
    }
    
    public void resetKill() {
        if (killMode == PRIMED) killMode = RESET;
    }
    
    private void pickPkg(int id) {
        Iterator pkgIter = localPkgs.iterator();
        //System.out.println("->"+localPkgs.size());
        while (pkgIter.hasNext()){
            GamePkg avail = (GamePkg) pkgIter.next();
            if (avail.getId() == id) {
                myPkgs.add(avail);
                pkgIter.remove();
                capacity = capacity - avail.getWeight();
            }
        }
    }
    
    private void dropPkg(int id) {
        Iterator pkg = myPkgs.iterator();
        while (pkg.hasNext()){
            GamePkg drop = (GamePkg) pkg.next();
            if (drop.getId() == id) {
                pkg.remove();
                capacity = capacity + drop.getWeight();
            }
        }
    }
    
    public char backward() {
        switch (this.lastDirection) {
            case MapSolver.EAST:
                return MapSolver.WEST;
            case MapSolver.WEST:
                return MapSolver.EAST;
            case MapSolver.NORTH:
                return MapSolver.SOUTH;
            case MapSolver.SOUTH:
                return MapSolver.NORTH;
            case 'X':
            default:
                return 'X';
        }
    }
                
                
    public void dump () throws BadLocationException {
        super.dump();
        System.out.println(this.capacity+" "+this.budget);
        if (myPkgs == null) return;
        
        Iterator i = myPkgs.iterator();
        while (i.hasNext()) {
            GamePkg pkg = (GamePkg) i.next();
            System.out.print(pkg.getId()+" ");
            pkg.getLocation().dump();
        }
            
    }
    
}
