/*
 * RobotServer.java
 *
 * Created on August 30, 2002, 11:11 PM
 */
import java.io.*;
import java.util.*;

/**
 *
 * @author  scerruti
 */
public class RobotServer extends java.net.Socket {
    
    private PrintWriter out = null;
    
    private BufferedReader in = null;
        
    private boolean connectionOpen = false;
    
    private GameMap map = null;

    private String currentCommand = null;
    
    private String[] segments = null;
    
    private int numSegments = -1;

    private int currentSegment = -1;
    
    private int currentRobot = -1;
    
    private MyPkgRobot myRobot;
    
    
    
    
    /** Creates a new instance of RobotServer */
    public RobotServer(String server, int port) throws IOException {
        super(server, port);
        out = new PrintWriter(this.getOutputStream(), true);
        in  = new BufferedReader(new InputStreamReader(this.getInputStream()));
        connectionOpen=true;
    }
    
    boolean connectionOpen() {
        return (connectionOpen);
    }
    
    void startGame() {
        out.println("Player");
    }
    
    void issueCommand(String command) {
        out.println(command);
        //System.out.println("Sent command:"+command);
    }
    
    GameMap getMap() throws IOException,BadLocationException,BadTileTypeException {
        
        this.parseCommand();
        
        int width = getNextInt();
        int height = getNextInt();
        
        //System.out.println(this.toString()+": map dimensions ("+width+","+height+")");
        map = new GameMap(width, height);
        for (int i = 1; i <= height; i++) {
            map.addLine(i, in.readLine());
        }
        
        return (map);
    }
    
    MyPkgRobot getMyRobot() throws IOException {
        
        this.parseCommand();
        
        int id       = getNextInt();
        int capacity = getNextInt();
        int budget   = getNextInt();
        return (this.myRobot = new MyPkgRobot(id, capacity, budget));
    }
    
    HashSet getRobotLocations() throws IOException, BadLocationException {
        this.parseCommand();
        HashSet robots = new HashSet();
        RobotServerCommand rsc;
        
        while ((rsc = this.parseGameStatus()) != null) {          
            if (rsc.command == RobotServerCommand.SET_LOCATION) {
                PkgRobot robot = new PkgRobot(map,rsc.id,rsc.x,rsc.y);
                robots.add(robot);
            }
        }
        return (robots);
    }
    
    TreeSet getPkgs() throws IOException, BadLocationException {
        int id;
        this.parseCommand();
        TreeSet pkgs = 
            new TreeSet(new PkgSorter(((PkgRobot) myRobot).getLocation()));
        
        if (this.numSegments < 4) {
            map.clearBase(((PkgRobot) myRobot).getLocation());
            return (null);
        }
        
        map.addBase(((PkgRobot) myRobot).getLocation());
        while ((id = this.getNextInt()) != -1) {
            int x=this.getNextInt();
            int y=this.getNextInt();
            int weight=this.getNextInt();
            pkgs.add(new GamePkg(map,id,x,y,weight));
            //System.out.println("# "+id+" "+pkgs.size());
        }
        
        return (pkgs);
    }
    
    ArrayList getResponse() throws IOException {
        this.parseCommand();
        ArrayList robotsStatus = new ArrayList();
        RobotServerCommand rsc;
        
        while ((rsc = this.parseGameStatus()) != null) {          
            robotsStatus.add(rsc);
        }
        return (robotsStatus);
    }
    
    void parseCommand() throws IOException {
        int pos        = 0;
        char delimiter = ' ';

        this.currentCommand = in.readLine();
        //System.out.println("From Server: "+this.currentCommand);
        //if (myRobot != null && myRobot.getLocation() != null) myRobot.getLocation().dump();
        if (this.currentCommand.indexOf("Robot ") != -1) {
            if (this.robotDied()) {
                this.numSegments=0;
                this.currentSegment=0;
                return;
            }
        }
        this.numSegments=1;
        this.currentSegment=0;

        pos=0;
        while ((pos=currentCommand.indexOf(delimiter,pos)) != -1) {
            //System.out.println(this.toString()+": delimiter pos "+pos);
            numSegments++;
            pos++;
        }
        
        segments = new String[numSegments];
        
        pos=0;
        for (int i = 0; i < numSegments; i++) {
            int end_pos=currentCommand.indexOf(delimiter,(pos+1));
            if (end_pos == -1) {
                segments[i] = currentCommand.substring(pos);
            } else {
                segments[i] = currentCommand.substring(pos,end_pos);
                pos=end_pos+1;
            }
        }
        //System.out.println(this.toString()+": "+currentCommand+" has "+numSegments+" segments");
    }
    
    String getNextSegment() {
        if (currentSegment < numSegments) {
            //System.out.println(this.toString()+": segment "+currentSegment+" is "+segments[currentSegment]);
            return (segments[currentSegment++]);
        } else {
            return null;
        }
    }
    
    int getNextInt() {
        String seg = getNextSegment();
        
        if (seg != null) {
            //System.out.println(this.toString()+": segment "+currentSegment+" is int "+Integer.parseInt(seg));
            return (Integer.parseInt(seg));
        } else {
            return -1;
        }
    }
    
    RobotServerCommand parseGameStatus() {
        RobotServerCommand rsc = new RobotServerCommand();
        
        rsc.command = RobotServerCommand.NO_COMMAND;
        
        if (currentSegment >= numSegments) {
            return null;
        }
        
        String segment = getNextSegment();
        
        if (segment.charAt(0) == '#') {
            currentRobot = Integer.parseInt(segment.substring(1));
            rsc.id = currentRobot;
            
            segment = getNextSegment();
            if (segment == null) return rsc;
            
            if (segment.charAt(0) == '#') {
                currentSegment--;
                return rsc;
            }
        }
        
        if (currentRobot == -1) {
            return null;
        }
        
        rsc.id = currentRobot;
        
        switch (segment.charAt(0)) {
            case 'E':
                rsc.command = RobotServerCommand.MOVE_EAST;
                break;
            case 'W':
                rsc.command = RobotServerCommand.MOVE_WEST;
                break;
            case 'N':
                rsc.command = RobotServerCommand.MOVE_NORTH;
                break;
            case 'S':
                rsc.command = RobotServerCommand.MOVE_SOUTH;
                break;
            case 'X':
                rsc.x = getNextInt();
                segment = getNextSegment();
                if (segment.charAt(0) != 'Y') break;
                rsc.y = getNextInt();
                rsc.command = RobotServerCommand.SET_LOCATION;
                break;
            case 'P':
                rsc.pkg = getNextInt();
                rsc.command = RobotServerCommand.PICK_PACKAGE;
                break;
            case 'D':
                rsc.pkg = getNextInt();
                rsc.command = RobotServerCommand.DROP_PACKAGE;
                break;
            default:
                rsc = null;
                break;
        }
                
        if (currentSegment >= numSegments) {
            currentRobot = -1;
        }
        
        return rsc;
    }
    
    private boolean robotDied() {
        String input = null;
        System.out.println(currentCommand);
        do {
            try {
                input = in.readLine();
                if (input != null) System.out.println(input);
            } catch (IOException e) {
            }
        } while (input != null);

        try {
            in.close();
            out.close();
            super.close();
            connectionOpen=false;
        } catch (IOException e1) {
            System.err.println("Error closing connection: "+e1.getMessage());
        }

        return (true);
    }

    
    private class PkgSorter implements Comparator {
        private GameBoardLocation loc;
        
        public PkgSorter(GameBoardLocation loc) {
            super();
            this.loc = loc;
        }
        
        public int compare(Object o1, Object o2) {
            int distance1 = loc.distanceTo(((GamePkg) o1).getLocation());
            int distance2 = loc.distanceTo(((GamePkg) o2).getLocation());
            if (distance1 != distance2) {
                return (distance2 - distance1);
            } else if (((GamePkg) o1).getWeight() != ((GamePkg) o2).getWeight()) {
                return (((GamePkg) o1).getWeight() - ((GamePkg) o2).getWeight());
            } else {
                return (((GamePkg) o2).getId() - ((GamePkg) o1).getId());
            }
        }
        
    }

}
